@@ -1,6 +1,6 @@
 /* pk-utils.c - Common utility functions for poke.  */
 
-/* Copyright (C) 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,6 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <sys/stat.h>
-#include <gettext.h>
-#define _(str) dgettext (PACKAGE, str)
 #include <stdarg.h> /* va_... */
 #include <stddef.h> /* size_t */
 #include <string.h> /* strcpy */
@@ -41,20 +39,20 @@ pk_file_readable (const char *filename)
   if (0 != stat (filename, &statbuf))
     {
       char *why = strerror (errno);
-      snprintf (errmsg, 4096, _("Cannot stat %s: %s\n"), filename, why);
+      snprintf (errmsg, 4096, "Cannot stat %s: %s\n", filename, why);
       return errmsg;
     }
 
   if (S_ISDIR (statbuf.st_mode))
     {
-      snprintf (errmsg, 4096, _("%s is a directory\n"), filename);
+      snprintf (errmsg, 4096, "%s is a directory\n", filename);
       return errmsg;
     }
 
   if (access (filename, R_OK) != 0)
     {
       char *why = strerror (errno);
-      snprintf (errmsg, 4096, _("%s: file cannot be read: %s\n"),
+      snprintf (errmsg, 4096, "%s: file cannot be read: %s\n",
                 filename, why);
       return errmsg;
     }
@@ -86,46 +84,50 @@ PK_POW (pk_upow, uint64_t)
 
 void
 pk_print_binary (void (*puts_fn) (const char *str),
-                 uint64_t val, int size, int sign)
+                 uint64_t val, int size, int sign_p, int use_suffix_p)
 {
   char b[65];
 
-  for (int z = 0; z < size; z++) {
+  for (int z = 0; z < size; z++)
     b[size-1-z] = ((val >> z) & 0x1) + '0';
-  }
   b[size] = '\0';
 
   puts_fn (b);
 
-  if (size == 64)
-    puts_fn (sign ? "L" : "UL");
-  else if (size == 16)
-    puts_fn (sign ? "H" : "UH");
-  else if (size == 8)
-    puts_fn (sign ? "B" : "UB");
-  else if (size == 4)
-    puts_fn (sign ? "N" : "UN");
+  if  (use_suffix_p)
+    {
+      if (size == 64)
+        puts_fn (sign_p ? "L" : "UL");
+      else if (size == 16)
+        puts_fn (sign_p ? "H" : "UH");
+      else if (size == 8)
+        puts_fn (sign_p ? "B" : "UB");
+      else if (size == 4)
+        puts_fn (sign_p ? "N" : "UN");
+    }
 }
 
 int
 pk_format_binary (char* out, size_t outlen,
-                  uint64_t val, int size, int sign)
+                  uint64_t val, int size, int sign_p, int use_suffix_p)
 {
   char b[64 /* digits */ + 2 /* suffix */ + 1 /* nul */];
 
-  for (int z = 0; z < size; z++) {
+  for (int z = 0; z < size; z++)
     b[size-1-z] = ((val >> z) & 0x1) + '0';
-  }
   b[size] = '\0';
 
-  if (size == 64)
-    strcat (b, sign ? "L" : "UL");
-  else if (size == 16)
-    strcat (b, sign ? "H" : "UH");
-  else if (size == 8)
-    strcat (b, sign ? "B" : "UB");
-  else if (size == 4)
-    strcat (b, sign ? "N" : "UN");
+  if (use_suffix_p)
+    {
+      if (size == 64)
+        strcat (b, sign_p ? "L" : "UL");
+      else if (size == 16)
+        strcat (b, sign_p ? "H" : "UH");
+      else if (size == 8)
+        strcat (b, sign_p ? "B" : "UB");
+      else if (size == 4)
+        strcat (b, sign_p ? "N" : "UN");
+    }
 
   if (strlen (b) < outlen)
     {
