@@ -1,28 +1,31 @@
 /* JitterLisp: primitives.
 
-   Copyright (C) 2017, 2018, 2019, 2020 Luca Saiu
+   Copyright (C) 2017, 2018, 2019, 2020, 2021 Luca Saiu
    Written by Luca Saiu
 
    This file is part of the JitterLisp language implementation, distributed as
-   an example along with Jitter under the same license.
+   an example along with GNU Jitter under the same license.
 
-   Jitter is free software: you can redistribute it and/or modify
+   GNU Jitter is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   Jitter is distributed in the hope that it will be useful,
+   GNU Jitter is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with Jitter.  If not, see <http://www.gnu.org/licenses/>. */
+   along with GNU Jitter.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #ifndef JITTERLISP_PRIMITIVES_H_
 #define JITTERLISP_PRIMITIVES_H_
 
+/* Include the Gnulib header. */
+#include <config.h>
+
 #include "jitterlisp-primitives.h"
 
 #include <string.h>
@@ -389,7 +392,16 @@ JITTERLISP_PRIMITIVE_FUNCTION_2_(plus, FIXNUM, FIXNUM,
 JITTERLISP_PRIMITIVE_FUNCTION_2_(minus, FIXNUM, FIXNUM,
   { JITTERLISP_MINUS_(res, args [0], args [1]); })
 JITTERLISP_PRIMITIVE_FUNCTION_2_(times, FIXNUM, FIXNUM,
-  { JITTERLISP_TIMES_(res, args [0], args [1]); })
+  { /* FIXME: do this for the other operations as well.  Factor. */
+    /* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
+    jitter_int arg0 = args [0];
+    jitter_int arg1 = args [1];
+    if (JITTER_WOULD_TIMES_OVERFLOW (jitter_uint,
+                                     jitter_int,
+                                     arg0, JITTERLISP_FIXNUM_DECODE (arg1),
+                                     JITTER_WORD_BIT_NO))
+      jitterlisp_error_cloned ("primordial-* overflow");
+    JITTERLISP_TIMES_(res, args [0], args [1]); })
 JITTERLISP_PRIMITIVE_FUNCTION_2_(divided, FIXNUM, FIXNUM,
   { JITTERLISP_DIVIDED_(res, args [0], args [1]); })
 JITTERLISP_PRIMITIVE_FUNCTION_2_(divided_unsafe, FIXNUM, FIXNUM,
@@ -639,22 +651,26 @@ JITTERLISP_PRIMITIVE_FUNCTION_1_(compiled_closure_print, COMPILED_CLOSURE,
 JITTERLISP_PRIMITIVE_FUNCTION_1_(compiled_closure_disassemble, COMPILED_CLOSURE,
   { JITTERLISP_COMPILED_CLOSURE_DISASSEMBLE_(res, args [0]); })
 /* Operations for handling debugging and profiling information. */
+JITTERLISP_PRIMITIVE_FUNCTION_0_(print_defects,
+  { jitterlispvm_defect_print (jitterlisp_print_context, 0); })
+JITTERLISP_PRIMITIVE_FUNCTION_0_(print_defect_replacements,
+  { jitterlispvm_defect_print_replacement_table (jitterlisp_print_context, 0); })
 JITTERLISP_PRIMITIVE_FUNCTION_0_(print_locations,
   { jitterlispvm_dump_data_locations (jitterlisp_print_context); })
 JITTERLISP_PRIMITIVE_FUNCTION_0_(print_profile_specialized,
   { struct jitterlispvm_profile_runtime *pr
-      = jitterlispvm_state_profile_runtime (& jitterlispvm_state);
+      = jitterlisp_current_profile_runtime ();
     jitterlispvm_profile_runtime_print_specialized (jitterlisp_print_context,
-                                                    pr); })
+                                                    pr);
+    jitterlispvm_profile_runtime_destroy (pr); })
 JITTERLISP_PRIMITIVE_FUNCTION_0_(print_profile_unspecialized,
   { struct jitterlispvm_profile_runtime *pr
-      = jitterlispvm_state_profile_runtime (& jitterlispvm_state);
+      = jitterlisp_current_profile_runtime ();
     jitterlispvm_profile_runtime_print_unspecialized (jitterlisp_print_context,
-                                                      pr); })
+                                                      pr);
+    jitterlispvm_profile_runtime_destroy (pr); })
 JITTERLISP_PRIMITIVE_FUNCTION_0_(reset_profileb,
-  { struct jitterlispvm_profile_runtime *pr
-      = jitterlispvm_state_profile_runtime (& jitterlispvm_state);
-    jitterlispvm_profile_runtime_clear (pr); })
+  { jitterlisp_reset_profile_runtime (); })
 /* Operations to display legal notices. */
 JITTERLISP_PRIMITIVE_FUNCTION_0_(copying,
   { jitter_print_char_star (jitterlisp_print_context, jitterlisp_gpl); })
@@ -902,6 +918,10 @@ jitterlisp_primitives []
       JITTERLISP_PRIMITIVE_PROCEDURE_STRUCT_("compiled-closure-disassemble", 1,
                                              compiled_closure_disassemble),
       /* Operations for handling debugging and profiling information. */
+      JITTERLISP_PRIMITIVE_PROCEDURE_STRUCT_("print-defects", 0,
+                                             print_defects),
+      JITTERLISP_PRIMITIVE_PROCEDURE_STRUCT_("print-defect-replacements", 0,
+                                             print_defect_replacements),
       JITTERLISP_PRIMITIVE_PROCEDURE_STRUCT_("print-locations", 0,
                                              print_locations),
       JITTERLISP_PRIMITIVE_PROCEDURE_STRUCT_("print-profile-specialized", 0,
