@@ -1,22 +1,22 @@
 /* Jitter: VM-independent library.
 
-   Copyright (C) 2016, 2017, 2019, 2020, 2021 Luca Saiu
+   Copyright (C) 2016, 2017, 2019, 2020, 2021, 2022 Luca Saiu
    Written by Luca Saiu
 
-   This file is part of Jitter.
+   This file is part of GNU Jitter.
 
-   Jitter is free software: you can redistribute it and/or modify
+   GNU Jitter is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   Jitter is distributed in the hope that it will be useful,
+   GNU Jitter is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with Jitter.  If not, see <http://www.gnu.org/licenses/>. */
+   along with GNU Jitter.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #ifndef JITTER_H_
@@ -67,28 +67,48 @@
    signed and unsigned; also define the format strings, JITTER_PRIi ,
    JITTER_PRIu , JITTER_PRIo and JITTER_PRIx to be used with printf and scanf
    for those types, in the style of PRIi64 , PRIu64 , PRIo64 and PRIx64 . */
-#if JITTER_WORD_BIT_NO == 16
+#if JITTER_WORD_CHAR_NO == JITTER_SIZEOF_SHORT
   /* This will not happen on GNU, but one check costs almost nothing. */
-  typedef int16_t  jitter_int;
-  typedef uint16_t jitter_uint;
-# define JITTER_PRIi PRIi16
-# define JITTER_PRIu PRIu16
-# define JITTER_PRIo PRIo16
-# define JITTER_PRIx PRIx16
-#elif JITTER_WORD_BIT_NO == 32
-  typedef int32_t  jitter_int;
-  typedef uint32_t jitter_uint;
-# define JITTER_PRIi PRIi32
-# define JITTER_PRIu PRIu32
-# define JITTER_PRIo PRIo32
-# define JITTER_PRIx PRIx32
-#elif JITTER_WORD_BIT_NO == 64
-  typedef int64_t  jitter_int;
-  typedef uint64_t jitter_uint;
-# define JITTER_PRIi PRIi64
-# define JITTER_PRIu PRIu64
-# define JITTER_PRIo PRIo64
-# define JITTER_PRIx PRIx64
+  typedef short  jitter_int;
+  typedef unsigned short  jitter_uint;
+# define JITTER_PRIi     "hi"
+# define JITTER_PRIu     "hu"
+# define JITTER_PRIo     "ho"
+# define JITTER_PRIx     "hx"
+# define JITTER_INT_MIN  SHRT_MIN
+# define JITTER_INT_MAX  SHRT_MAX
+# define JITTER_UINT_MAX USHRT_MAX
+#elif JITTER_WORD_CHAR_NO == JITTER_SIZEOF_INT
+  typedef int  jitter_int;
+  typedef unsigned int  jitter_uint;
+# define JITTER_PRIi     "i"
+# define JITTER_PRIu     "u"
+# define JITTER_PRIo     "o"
+# define JITTER_PRIx     "x"
+# define JITTER_INT_MIN  INT_MIN
+# define JITTER_INT_MAX  INT_MAX
+# define JITTER_UINT_MAX UINT_MAX
+#elif JITTER_WORD_CHAR_NO == JITTER_SIZEOF_LONG
+  typedef long  jitter_int;
+  typedef unsigned long  jitter_uint;
+# define JITTER_PRIi     "li"
+# define JITTER_PRIu     "lu"
+# define JITTER_PRIo     "lo"
+# define JITTER_PRIx     "lx"
+# define JITTER_INT_MIN  LONG_MIN
+# define JITTER_INT_MAX  LONG_MAX
+# define JITTER_UINT_MAX ULONG_MAX
+#elif defined (JITTER_HAVE_LONG_LONG_INT)                  \
+      && (JITTER_WORD_CHAR_NO == JITTER_SIZEOF_LONG_LONG)
+  typedef long long  jitter_int;
+  typedef unsigned long long  jitter_uint;
+# define JITTER_PRIi     "lli"
+# define JITTER_PRIu     "llu"
+# define JITTER_PRIo     "llo"
+# define JITTER_PRIx     "llx"
+# define JITTER_INT_MIN  LLONG_MIN
+# define JITTER_INT_MAX  LLONG_MAX
+# define JITTER_UINT_MAX ULLONG_MAX
 #else
 # error "can't find a word-sized integer type."
 #endif // #if    (JITTER_SIZEOF_VOID_P == ...)
@@ -103,7 +123,8 @@
   typedef float jitter_float;
 #elif  (JITTER_SIZEOF_VOID_P == JITTER_SIZEOF_DOUBLE)
   typedef double jitter_float;
-#elif  (JITTER_SIZEOF_VOID_P == JITTER_SIZEOF_LONG_DOUBLE)
+#elif defined (HAVE_LONG_DOUBLE) \
+      && (JITTER_SIZEOF_VOID_P == JITTER_SIZEOF_LONG_DOUBLE)
 /* This should not happen anywhere, I guess -- on PowerPC long double is 64-bit
    but so is double which is checked before.  Anyway, it costs nothing. */
   typedef long double jitter_float;
@@ -132,8 +153,10 @@ typedef jitter_int jitter_label_as_index;
    value without loss of information, given the appropriate signedness; in this
    project enum values are always non-negative.
 
-   FIXME: is this design a problem on m68k, which has different registers for
-   integers and addresses?  Not very high-priority, but I'm curious. */
+   The m68k architecture is unusual because of its two different classes of
+   registers for integers and addresses.  GCC manages to do a good job even on
+   m68k, not always using only one class of registers for unions of this
+   type. */
 union jitter_word
 {
   /* A signed word-sized integer. */
@@ -163,6 +186,41 @@ union jitter_word
 };
 
 
+
+
+/* Reinterpret-cast conversion.
+ * ************************************************************************** */
+
+/* Include the machinery to be used in the next section. */
+#include <jitter/jitter-reinterpret-cast.h>
+
+
+
+
+/* Reinterpret-cast conversion between floating-point and integer types.
+ * ************************************************************************** */
+
+/* Given a jitter_uint expression expand to an expression evaluating to the same
+   expression and then reinterpret-cast to jitter_float. */
+#define JITTER_REINTERPRET_CAST_UINT_TO_FLOAT(from)            \
+  JITTER_REINTERPRET_CAST (jitter_float, jitter_uint, (from))
+
+/* Given a jitter_int expression expand to an expression evaluating to the same
+   expression and then reinterpret-cast to jitter_float. */
+#define JITTER_REINTERPRET_CAST_INT_TO_FLOAT(from)            \
+  JITTER_REINTERPRET_CAST (jitter_float, jitter_int, (from))
+
+/* Given a jitter_float expression expand to an expression evaluating to the
+   same expression and then reinterpret-cast to jitter_uint. */
+#define JITTER_REINTERPRET_CAST_FLOAT_TO_UINT(from)            \
+  JITTER_REINTERPRET_CAST (jitter_uint, jitter_float, (from))
+
+/* Given a jitter_float expression expand to an expression evaluating to the
+   same expression and then reinterpret-cast to jitter_int. */
+#define JITTER_REINTERPRET_CAST_FLOAT_TO_INT(from)            \
+  JITTER_REINTERPRET_CAST (jitter_int, jitter_float, (from))
+
+
 
 
 /* The long long type or some approximation of it.
@@ -233,7 +291,7 @@ union jitter_word
 
 
 
-/* Definitions depending on the dispatch model.
+/* Definitions depending on the dispatch.
  * ************************************************************************** */
 
 /* Sanity check: make sure that the user passed the correct set of CPP flags.  
@@ -274,9 +332,9 @@ JITTER_CPPFLAGS or one of its variants as intended will prevent subtle \
 problems.  See the source code for more information."
 #endif // no JITTER_CPPFLAGS or -DJITTER_INTERNAL.
 
-/* Check that one dispatching model is defined with a CPP macro, and define
+/* Check that one dispatch is defined with a CPP macro, and define
    JITTER_REPLICATE if needed.  Also define the JITTER_DISPATCH_NAME and
-   JITTER_DISPATCH_NAME_STRING macros as the dispatching model name, to be
+   JITTER_DISPATCH_NAME_STRING macros as the dispatch name, to be
    respectively used as a C identifier and as text for user messages. */
 #if   defined(JITTER_DISPATCH_SWITCH)
 # define JITTER_DISPATCH_NAME        switch
@@ -295,12 +353,12 @@ problems.  See the source code for more information."
   /* No-threading requires code replication. */
 # define JITTER_REPLICATE 1
 #elif ! defined (JITTER_INTERNAL)
-# error "unknown dispatching model.  This should never happen."
+# error "unknown dispatch.  This should never happen."
 #endif // #if defined(JITTER_DISPATCH_...)
 
-/* Sanity check: do not use advanced dispatching models on a system missing
+/* Sanity check: do not use advanced dispatches on a system missing
    prerequisites.  It is friendlier to fail here than with some mysterious
-   compilation error later. 
+   compilation error later.
    It is more helpful to have separate checks, so that the user knows what
    the exact missing requirement is. */
 #if (defined (JITTER_DISPATCH_MINIMAL_THREADING) \
@@ -329,11 +387,11 @@ problems.  See the source code for more information."
 
 
 
-/* The selected dispatching model name as a C identifier.
+/* The selected dispatch name as a C identifier.
  * ************************************************************************** */
 
-/* Compute the name of a C global whose name depends on the dispatching model.
-   Only the one for the selected model is defined, and this serves to prevent
+/* Compute the name of a C global whose name depends on the dispatch.
+   Only the one for the selected dispatch is defined, and this serves to prevent
    mistakes when linking jitterc-generated code to a runtime library; the
    definition is in jitter.c and the code using the global is in vm1.c . */
 #define JITTER_DISPATCH_DEPENDENT_GLOBAL_NAME                \
@@ -342,6 +400,57 @@ problems.  See the source code for more information."
                            _dispatch)
 
 
+
+
+/* Program points.
+ * ************************************************************************** */
+
+/* The type of a program point at run time in an executable routine.  This is
+   the type of object than can be passed to JITTER_BRANCH , and its actual
+   definition depends on the dispatch.  Notice that however, in every
+   case, a program point is a pointer-to-constant type and therefore fits in a
+   word. */
+#ifdef JITTER_DISPATCH_NO_THREADING
+  /* With no-threading dispatch a program point is the address of a machine
+     instruction -- from C, it's what a goto * statement accepts.  I don't
+     need to worry about non-GCC compilers, since no-threading relies on GCC
+     extensions. */
+  typedef const void *
+  jitter_program_point;
+#else
+  /* On every other dispatch a program point is a pointer to a word
+     in the executable routine -- in the case of switch dispatching that word
+     will contain a specialized opcode, with threading it will contain the
+     address of a machine instruction (see the case above) followed by the
+     VM instruction arguments. */
+  typedef const union jitter_word *
+  jitter_program_point;
+#endif // #ifdef JITTER_DISPATCH_NO_THREADING
+
+/* Expand to an expression evaluating to the program point of the first
+   instruction in the pointed executable routine, as some object which is
+   correct to pass to JITTER_BRANCH.  The expression type will be
+   jitter_program_point.
+
+   This macro is usable from within VM instructions, which might for example
+   perform a jump to the beginning of a different VM routine.  The expansion of
+   this macro is guaranteed not to contain function calls, and is safe to use
+   without extra C function wrappers. */
+#ifdef JITTER_DISPATCH_NO_THREADING
+  /* In this case the executable routine contains a separate pointer to the
+     beginning of the executable region for the native routine.  Of course
+     the first instruction is at the beginning of the region. */
+# define JITTER_EXECUTABLE_ROUTINE_BEGINNING(_jitter_executable_routine_ptr)  \
+    ((_jitter_executable_routine_ptr)->native_code)
+#else
+  /* With switch dispatching or threading the first program point is a pointer
+     to the beginning of the specialized routine array. */
+# define JITTER_EXECUTABLE_ROUTINE_BEGINNING(_jitter_executable_routine_ptr)  \
+    ((jitter_program_point)                                                   \
+     ((_jitter_executable_routine_ptr)->specialized_routine))
+#endif // ifdef JITTER_DISPATCH_NO_THREADING
+
+
 
 
 /* Feature macros derived on other macros.
@@ -349,7 +458,7 @@ problems.  See the source code for more information."
 
 /* The following macros are useful from many places in C code, in particular in
    order to know whether some feature should be enabled, depending on the
-   dispatching mode or the architecture.
+   dispatch or the architecture.
    It is convenient to define them here in a centralized way, rather than in
    individual headers which the user may forget to include before testing
    whether some feature macro is defined.
@@ -358,7 +467,7 @@ problems.  See the source code for more information."
    - Feature macros defining whether a functionality could be used have names
      starting with "JITTER_HAVE_" .
    - Feature macros defining whether a functionality is actually used (in the
-     current dispatching mode, for the program which is being compiled) have
+     current dispatch, for the program which is being compiled) have
      names starting with "JITTER_USE_" .
    Notice that the feature macros defined in jitter/jitter-config.h are also
    visible from here. */
@@ -390,4 +499,4 @@ problems.  See the source code for more information."
 # include <jitter/machine/jitter-machine.h>
 #endif // #ifdef JITTER_ENABLE_ASSEMBLY
 
-#endif // #ifndef JITTER_CONFIG_H_
+#endif // #ifndef JITTER_H_
