@@ -571,7 +571,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_op_mul)
       else if (PKL_AST_TYPE_CODE (op_type) == PKL_TYPE_STRING)
         ;
       else
-        assert (0);
+        PK_UNREACHABLE ();
     }
 
   PKL_PASS_DONE;
@@ -753,7 +753,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_op_bshiftpow)
         break;
       }
     default:
-      assert (0);
+      PK_UNREACHABLE ();
     }
 
   PKL_PASS_DONE;
@@ -977,7 +977,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_type_array)
         break;
       }
     default:
-      assert (0); /* This can't happen.  */
+      PK_UNREACHABLE (); /* This can't happen.  */
     }
 
   PKL_PASS_RESTART = restart;
@@ -1662,7 +1662,7 @@ PKL_PHASE_BEGIN_HANDLER (pkl_promo_ps_cons)
         break;
       }
     default:
-      assert (0);
+      PK_UNREACHABLE ();
     }
 
   PKL_PASS_DONE;
