@@ -23,6 +23,7 @@
 #include <stdio.h>
 #include <stdarg.h>
 
+#include "pkl-ast.h"
 #include "pkl-compiler.h"
 #include "pvm.h"
 
@@ -74,6 +75,9 @@
    RT_PATH should contain the name of a directory where the compiler can
    find its run-time support files.
 
+   CONFIG_PATH should contain the name of a directoy where
+   pkl-config.pk can be found.
+
    FLAGS is an ORed value of PKL_F_* values, or 0.
 
    If there is an error creating the compiler this function returns
@@ -81,7 +85,8 @@
 
 #define PKL_F_NOSTDTYPES 1 /* Do not define standard types.  */
 
-pkl_compiler pkl_new (pvm vm, const char *rt_path, uint32_t flags);
+pkl_compiler pkl_new (pvm vm, const char *rt_path,
+                      const char *config_path, uint32_t flags);
 
 void pkl_free (pkl_compiler compiler);
 
@@ -308,4 +313,9 @@ int pkl_module_loaded_p (pkl_compiler compiler, const char *path);
 
 void pkl_add_module (pkl_compiler compiler, const char *path);
 
+/* Given an AST node, run constant fold on it.  */
+
+pkl_ast_node pkl_constant_fold (pkl_compiler compiler, pkl_ast ast,
+                                pkl_ast_node node);
+
 #endif /* ! PKL_H */
