@@ -1,6 +1,6 @@
 /* pk-cmd-def.c - commands related to definitions.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,9 +51,10 @@ print_var_decl (int kind,
   struct pk_info_payload *payload = (struct pk_info_payload *) data;
   pk_table table = payload->table;
   regex_t regexp = payload->regexp;
+  regmatch_t match;
 
   if (payload->regexp_p
-      && regexec (&regexp, name, 1, NULL, 0) != 0)
+      && regexec (&regexp, name, 1, &match, 0) != 0)
     return;
 
   pk_table_row (table);
@@ -83,9 +84,10 @@ print_fun_decl (int kind,
   struct pk_info_payload *payload = (struct pk_info_payload *) data;
   pk_table table = payload->table;
   regex_t regexp = payload->regexp;
+  regmatch_t match;
 
   if (payload->regexp_p
-      && regexec (&regexp, name, 1, NULL, 0) != 0)
+      && regexec (&regexp, name, 1, &match, 0) != 0)
     return;
 
   pk_table_row (table);
@@ -114,9 +116,10 @@ print_type_decl (int kind,
   struct pk_info_payload *payload = (struct pk_info_payload *) data;
   pk_table table = payload->table;
   regex_t regexp = payload->regexp;
+  regmatch_t match;
 
   if (payload->regexp_p
-      && regexec (&regexp, name, 1, NULL, 0) != 0)
+      && regexec (&regexp, name, 1, &match, 0) != 0)
     return;
 
   pk_table_row (table);
@@ -165,10 +168,10 @@ print_type_decl (int kind,
               return 0;                                                 \
             }                                                           \
           payload.regexp_p = 1;                                         \
+          payload.regexp = regexp;                                      \
         }                                                               \
       else                                                              \
         payload.regexp_p = 0;                                           \
-      payload.regexp = regexp;                                          \
     }                                                                   \
   while (0)
 
