@@ -334,7 +334,7 @@ enum yysymbol_kind_t
   YYSYMBOL_YYEOF = 0,                      /* "end of file"  */
   YYSYMBOL_YYerror = 1,                    /* error  */
   YYSYMBOL_YYUNDEF = 2,                    /* "invalid token"  */
-  YYSYMBOL_ERROR = 3,                      /* ERROR  */
+  YYSYMBOL_ERR = 3,                        /* ERR  */
   YYSYMBOL_SEP = 4,                        /* SEP  */
   YYSYMBOL_ENTRY = 5,                      /* ENTRY  */
   YYSYMBOL_TAG = 6,                        /* TAG  */
@@ -744,8 +744,8 @@ static const char *yysymbol_name (yysymbol_kind_t yysymbol) YY_ATTRIBUTE_UNUSED;
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
 {
-  "\"end of file\"", "error", "\"invalid token\"", "ERROR", "SEP",
-  "ENTRY", "TAG", "DATA", "$accept", "map", "map_entries", "map_entry",
+  "\"end of file\"", "error", "\"invalid token\"", "ERR", "SEP", "ENTRY",
+  "TAG", "DATA", "$accept", "map", "map_entries", "map_entry",
   "tagged_value_list", "tagged_value", YY_NULLPTR
 };
 
@@ -1732,7 +1732,7 @@ yyreduce:
                       else if (!offset)
                         pk_map_printf_error (map_parser, (yyloc), "entry lacks an %%offset\n");
                       else
-                        assert (0);
+                        PK_UNREACHABLE ();
 
                       YYERROR;
                     }
